@@ -19,8 +19,6 @@
 # Author: Aurelien Bompard <abompard@fedoraproject.org>
 #
 
-from __future__ import absolute_import, unicode_literals
-
 from django.apps import AppConfig
 
 
@@ -31,3 +29,4 @@ class HyperKittyConfig(AppConfig):
     def ready(self):
         # flake8: noqa
         import hyperkitty.signals
+        import hyperkitty.checks
